﻿PageLines = function () {

	var pollingTimer;
	var elapsedTimer;
	var polling;
	var lastSyncIdentifier;
	var angularModuleCreated;
	var defaultSelectedSidebarId = "sidebarMenuAll";
	var defaultLinesSortKey = "line";
	var itemsMapping;
	var elapsedTimeAutoRefreshTimeoutInSeconds = 1;
	var lineAutoRefreshRates = [5, 10, 15, 20, 25, 30];
	var defaultLineAutoRefreshRate = 30;

	/**************************** Private functions ****************************/

	var init = function () {
		stopPolling();
		InitialiseForms(".nuggets");

		itemsMapping = {};

		var body = $('body');

		// Remove all body css classes.
		body.removeClass();

		// Apply classes.
		body.addClass('sidebar-left page-monitor');

		// Select menu.
		var menu = $('.monitor');
		menu.addClass('selected');

		// Set help link.
		HelpManual.setToMonitor();

		// Angular.
		var module = angularModuleInstance();
		AngularHelper.disableAngularRequestsCaching(module);
		createAngularDataFactory();
		createAngularNuggets();
		createAngularSidebar();
		createAngularHeader();
		createAngularLines();

		// Bind mouse events.
		var container = getContainer();
		container.mouseScroll({
			onScrollUp: onPreviousItems,
			onScrollDown: onNextItems
		});

		// Bind keyboard events.
		body.keyboardEvents({
			onPageUp: onPreviousItems,
			onArrowUp: onPreviousItems,
			onPageDown: onNextItems,
			onArrowDown: onNextItems
		});

		// Bind window events.
		var currentWindow = $(window);
		currentWindow.resize(onWindowResize);

		updateLinesElapsedTime();

		body = null;
		menu = null;
		container = null;
		currentWindow = null;
	};

	var onWindowResize = function (e) {
		var linesScope = getLinesScope();
		if (linesScope != null) {
			safeApply(linesScope, function () {
				// Set the maximum items which can fit in screen.
				linesScope.updateItemsLimit();
			});

			linesScope = null;
		}
	};

	var onPreviousItems = function (e) {
		var linesScope = getLinesScope();
		if (linesScope != null) {
			safeApply(linesScope, function () {
				linesScope.viewPrevItems();
			});

			linesScope = null;
		}
	};

	var onNextItems = function (e) {
		var linesScope = getLinesScope();
		if (linesScope != null) {
			safeApply(linesScope, function () {
				linesScope.viewNextItems();
			});

			linesScope = null;
		}
	};

	var angularModuleInstance = function () {
		var appName = "myApp";

		if (angularModuleCreated == null || !angularModuleCreated) {
			angularModuleCreated = true;
			return angular.module(appName, []);
		}
		else {
			return angular.module(appName);
		}
	};

	var createAngularDataFactory = function () {
		var module = angularModuleInstance();

		// Create a factory to get backend data.
		module.factory("dataFactory", ["$http", function ($http) {
			var dataFactory = {};

			dataFactory.getLines = function () {
				return $http({
					url: rootPath + "Monitor/GetLines",
					method: "GET",
					params: { lastSyncIdentifier: lastSyncIdentifier }
				});
			};

			dataFactory.restartService = function () {
				return $.ajax({
					type: "POST",
					dataType: "json",
					contentType: 'application/json; charset=utf-8',
					url: rootPath + "Monitor/RestartService"
				});
			};

			dataFactory.abortTransmission = function (lineId) {
				return $.ajax({
					type: "POST",
					dataType: "json",
					contentType: 'application/json; charset=utf-8',
					url: rootPath + "Monitor/AbortTransmission",
					data: JSON.stringify({ lineId: lineId })
				});
			};

			dataFactory.initialiseLine = function (lineId) {
			    return $.ajax({
			        type: "POST",
			        dataType: "json",
			        contentType: 'application/json; charset=utf-8',
			        url: rootPath + "Monitor/InitialiseLine",
			        data: JSON.stringify({ lineId: lineId })
			    });
			};

			return dataFactory;
		}]);

		module = null;
	};

	var createAngularNuggets = function () {
		var module = angularModuleInstance();

		// Create a controller to communicate with the view.
		module.controller("nuggetsController", ["$scope", function ($scope) {
			var refreshRatePermStorageKey = "page-lines-refresh-rate";

			// Init - Set default value.
			$scope.refreshRate = PermStorage.GetData(refreshRatePermStorageKey);
			if ($scope.refreshRate == null || isNaN($scope.refreshRate))
				$scope.refreshRate = defaultLineAutoRefreshRate;

			$scope.lineAutoRefreshRates = lineAutoRefreshRates;
			$scope.changeRefreshRate = function (value) {
				$scope.refreshRate = value;
				PermStorage.AddData(refreshRatePermStorageKey, value);
			};
		}]);

		module.directive('customSelectChange', function () {
			return {
				restrict: 'A',
				link: function (scope, element, attrs) {
					element.bind('change', function () {
						var fnChange = scope[attrs.customSelectChange];
						if (fnChange)
							fnChange(element.attr('data-value'));
					});
				},
			};
		});

		var controller = $("[ng-controller='nuggetsController']");
		angular.bootstrap(controller, [module.name]);

		module = null;
		controller = null;
	};

	var createAngularSidebar = function () {
		var module = angularModuleInstance();

		// Create a controller to communicate with the view.
		module.controller("sidebarController", ["$scope", function ($scope) {
			// Init - Set default value.
			$scope.items = [];

			$scope.items.push({ id: "sidebarMenuAll", parentStatus: LineParentStatus.ALL.value, name: LineParentStatus.ALL.name });
			$scope.items.push({ id: "sidebarMenuFree", parentStatus: LineParentStatus.FREE.value, name: LineParentStatus.FREE.name });
			$scope.items.push({ id: "sidebarMenuSending", parentStatus: LineParentStatus.OUTBOUND.value, name: LineParentStatus.OUTBOUND.name });
			$scope.items.push({ id: "sidebarMenuReceiving", parentStatus: LineParentStatus.INBOUND.value, name: LineParentStatus.INBOUND.name });
			$scope.items.push({ id: "sidebarMenuUninitialized", parentStatus: LineParentStatus.UNINITIALISED.value, name: LineParentStatus.UNINITIALISED.name });

			$scope.filterLinesByParentStatus = function (event, parentStatus) {
				var linesScope = getLinesScope();
				if (linesScope != null) {
					safeApply(linesScope, function () {
						linesScope.filterByParentStatus(parentStatus);
					});

					linesScope = null;
				}
			}
		}]);

		// Create sidebar directive when rendering completes.
		module.directive("renderSidebarComplete", ["$timeout", function ($timeout) {
			return function (scope, element, attr) {
				if (scope.$last === true) {
					$timeout(function () {
						autoUpdateRootCrumb('.monitor');

						var sidebarScope = getSidebarScope();
						if (sidebarScope != null) {
							// Filter sidebar items by id.
							var filteredViewItems = $.grep(sidebarScope.items, function (sidebarItem) {
								return sidebarItem.id == defaultSelectedSidebarId;
							});

							// Select sidebar item.
							var sidebarItem = $('#' + defaultSelectedSidebarId);
							var parent = sidebarItem.parent('li');
							selectSidebarItem(parent);

							if (filteredViewItems.length > 0) {
								scope.filterLinesByParentStatus(null, filteredViewItems[0].parentStatus);
							}

							sidebarScope = null;
							filteredViewItems = null;
							sidebarItem = null;
							parent = null;
						}
					});
				}
			};
		}]);

		var controller = $("[ng-controller='sidebarController']");
		angular.bootstrap(controller, [module.name]);

		module = null;
		controller = null;
	};

	var createAngularHeader = function () {
		var module = angularModuleInstance();

		// Create a controller to communicate with the view.
		module.controller("headerController", ["$scope", function ($scope) {
			// Init - Set default value.
			$scope.items = [];

			$scope.items.push({ cssClass: "icon", name: null, icon: null, sortKey: "parentStatus" });
			$scope.items.push({ cssClass: "faxLine", name: MonitorLineStrings.HeaderFaxLine, icon: "icon-pictos-570", sortKey: "line" });
			$scope.items.push({ cssClass: "device", name: MonitorLineStrings.HeaderDevice, icon: "icon-pictos-570", sortKey: "device" });
			$scope.items.push({ cssClass: "transmissionStatus", name: MonitorLineStrings.HeaderTransmissionStatus, icon: "icon-pictos-570", sortKey: "faxStartDateTime" });

			$scope.sortItems = function (event, sortKey) {
				var linesScope = getLinesScope();
				if (linesScope != null) {
					safeApply(linesScope, function () {
						linesScope.sortItems(sortKey);
					});

					linesScope = null;
				}
			}
		}]);

		var controller = $("[ng-controller='headerController']");
		angular.bootstrap(controller, [module.name]);

		module = null;
		controller = null;
	};

	var createAngularLines = function () {
		var module = angularModuleInstance();

		// Define a filter for items pagination.
		module.filter('itemsStartFrom', function () {
			return function (input, start) {
				start = +start; //parse to int
				return input.slice(start);
			}
		});

		// Create a controller to communicate with the view.
		module.controller("linesController", ["$scope", "dataFactory", function ($scope, dataFactory) {
			// Init - Set default value.
			$scope.items = [];
			$scope.startFrom = 0;
			$scope.itemsLimit = 0;
			$scope.isSortAsc;
			$scope.sortBy;

			// Get the next items from cache.
			$scope.viewNextItems = function () {
				var currentEndPosition = $scope.startFrom + $scope.itemsLimit - 1;

				// If there are more lines to be dispayed, display them.
				if (currentEndPosition < ($scope.items.length - 1)) {
					$scope.startFrom += 1;
				}
			}

			// Get the previous items from cache.
			$scope.viewPrevItems = function () {
				// If there are more lines to be dispayed, display them.
				if ($scope.startFrom > 0) {
					$scope.startFrom -= 1;
				}
			}

			// The maximum number of lines which can be displayed on screen.
			$scope.updateItemsLimit = function () {
				var container = getContainer();
				var rowHeight = 55;
				var pageSize = container.height() / rowHeight;

				$scope.itemsLimit = Math.ceil(pageSize);
			}

			// Filter lines by category.
			$scope.filterByParentStatus = function (parentStatus) {
				$scope.startFrom = 0;

				if (parentStatus == null) {
					$scope.matchParentStatus = null;
				}
				else {
					$scope.matchParentStatus = function (item) {
						return item.parentStatus == parentStatus;
					}
				}
			}

			// Get lines from backend.
			$scope.getLines = function () {
				return dataFactory.getLines();
			}

			// Add item in view.
			$scope.addItem = function (item) {
				$scope.items.push(item);

				// Create mapping.
				itemsMapping[item.id] = $scope.items.length - 1;
			}

			// Get fax line from view.
			$scope.getItem = function (lineId) {
				var pos = $scope.getItemPos(lineId);
				if (pos == null) {
					return null;
				}

				// Get view item.
				return $scope.items[pos];
			};

			// Get the array position of the fax line (which is in the view).
			$scope.getItemPos = function (lineId) {
				return itemsMapping[lineId];
			};

			// Create a mapping between a fax line id and it's array position.
			$scope.rebuildMappings = function () {
				itemsMapping = {};

				for (var i = 0; i < $scope.items.length; i++) {
					itemsMapping[$scope.items[i].id] = i;
				}
			}

			// Restart fax service.
			$scope.restartService = function (event) {
				var buttons = [
                    Notif.getButton(NotifStrings.Ok, "btn btn-ui-actions", function () {
                    	var jqXHR = dataFactory.restartService();

                    	Notif.close();

                    	jqXHR.success(function (response) {
                    		if (!response.success)
                    			Site.showError(MonitorLineStrings.getError(response.data));
                    	});
                    }),

                    Notif.getButton(NotifStrings.Cancel, "btn btn-ui-actions", Notif.close)
				];

				Notif.displayNotification(NotifStrings.Warning, "Warning", MonitorLineStrings.ModalRestartServerConfirm, buttons);

				buttons = null;
			};

			// Abort fax transmission.
			$scope.abortTransmission = function (event, lineId) {
				var buttons = [
                    Notif.getButton(NotifStrings.Ok, "btn btn-ui-actions", function () {
                    	var jqXHR = dataFactory.abortTransmission(lineId);

                    	jqXHR.success(function (response) {
                    		if (!response.success)
                    			Site.showError(MonitorLineStrings.getError(response.data));
                    		else
                    			Notif.close();
                    	});
                    }),

                    Notif.getButton(NotifStrings.Cancel, "btn btn-ui-actions", Notif.close)
				];

				Notif.displayNotification(NotifStrings.Warning, "Warning", MonitorLineStrings.ModalAbortFaxConfirm, buttons);

				buttons = null;
			}

			// Initialise line.
			$scope.initialiseLine = function (event, lineId) {
				var buttons = [
                    Notif.getButton(NotifStrings.Ok, "btn btn-ui-actions", function () {
                    	var jqXHR = dataFactory.initialiseLine(lineId);

                    	jqXHR.success(function (response) {
                    		if (!response.success)
                    			Site.showError(MonitorLineStrings.getError(response.data));
                    		else
                    			Notif.close();
                    	});
                    }),

                    Notif.getButton(NotifStrings.Cancel, "btn btn-ui-actions", Notif.close)
				];

				Notif.displayNotification(NotifStrings.Warning, "Warning", MonitorLineStrings.ModalInitialiseLineConfirm, buttons);

				buttons = null;
			}

		    // Sort view items.
			$scope.sortItems = function (key) {
				if (isNullOrEmpty(key)) {
					return;
				}

				$scope.startFrom = 0;

				if ($scope.isSortAsc == null)
					$scope.isSortAsc = true;
				if ($scope.sortBy == key)
					$scope.isSortAsc = !$scope.isSortAsc;
				$scope.sortBy = key;

				$scope.items.sort(sortByProperty(key, $scope.isSortAsc));
				$scope.rebuildMappings();
			}

			$scope.updateItemsLimit();
			$scope.sortItems(defaultLinesSortKey);
		}]);

		var controller = $("[ng-controller='linesController']");
		angular.bootstrap(controller, [module.name]);

		module = null;
		controller = null;
	};

	var startPolling = function () {
		polling = true;
		resetPolling();
	};

	var resetPolling = function () {
		clearTimeout(pollingTimer);

		pollingTimer = setTimeout(function () {
			updateUI(resetPolling);
		}, getNuggetsScope().refreshRate * 1000);
	};

	var stopPolling = function () {
		polling = false;
		clearTimeout(pollingTimer);
		pollingTimer = null;
	};

	var updateUI = function (onUpdateUIComplete) {
		var linesScope = getLinesScope();
		if (linesScope != null) {
			var jqXHR = linesScope.getLines();

			jqXHR.success(function (response) {
				// If cannot retreive lines.
				if (!response.success || response.data == null) {
					linesScope.items = [];

					// Rebuild mappings.
					linesScope.rebuildMappings();

					updateSidebarCounts(linesScope.items);

					if (onUpdateUIComplete != null) {
						onUpdateUIComplete();
					}

					lastSyncIdentifier = null;

					return;
				}

				var jsonObject = $.parseJSON(response.data);

				// Update breadcrumb - OutQueue part.
				$('#outQueueCount').html(jsonObject.outQueueSize);

				// Update breadcrumb - Converting part.
				var converterStatus = FaxEnum.getLineConverterStatusByValue(jsonObject.converterStatus);
				$('#converting').html((converterStatus != null) ? converterStatus.name : null);

				// Update breadcrumb - LatestFile part.
				$('#latestFile').html(jsonObject.lastFileProcessed);

				var viewItem;
				var status;
				var parentStatus;
				var resolution;
				var statusCss;
				var lastError;
				var parentStatusChanged;

				// Check for new or updated items.
				$.each(jsonObject.data, function (index, line) {
				    viewItem = null;
					status = null;
					parentStatus = null;
					resolution = null;
					statusCss = null;
					lastError = null;

					// Get view item.
					viewItem = linesScope.getItem(line.id);

					// New view item.
					if (viewItem == null) {
						viewItem = {};
						viewItem.id = line.id;
						linesScope.addItem(viewItem);
					}

					// Device name.
					viewItem.device = line.values.deviceName;

					// Line.
					//todo: check the relevance
					viewItem.channel = FaxEnum.getLineByValue(line.values.portName);

					// Speed.
					viewItem.speed = (line.values.speed != null) ? Site.formatString("{0} {1}", line.values.speed, MonitorLineStrings.Bps) : null;

					// Custom status.
					if (line.values.customStatus != null) {
						viewItem.statusName = line.values.customStatus;
					}
					else {
						// Status.
						status = FaxEnum.getLineStatusByValue(line.values.status);
						viewItem.statusName = (line.values.faxNumber != null) ? Site.formatString(status.name, line.values.faxNumber) : status.name;
					}

					//Field for correct sorting
					viewItem.line = parseInt(line.values.line);

					//Title
					viewItem.lineTitle = Site.formatString(MonitorLineStrings.FaxLineTitle, viewItem.line, line.values.portName);

					// Received bytes.
					viewItem.dataTransferred = (line.values.transferedBytes != null && status == LineStatus.RECEIVING) ? resolveStorageSize(line.values.transferedBytes) : null;

					// The elapsed time is calculated by the method "updateLinesElapsedTime".
					// Fax start date time.
					viewItem.faxStartDateTime = getFaxStartTime(jsonObject.currentTimestamp, line.values.startTime);
					// Elapsed time.
					viewItem.elapsedTime = DateTime.getElapsedTime(viewItem.faxStartDateTime);

					// Resolution.
					resolution = FaxEnum.getResolutionByValue(line.values.resolution);
					viewItem.resolution = (resolution != null) ? resolution.name : null;

					// Group.
					parentStatus = FaxEnum.getParentStatusByValue(line.values.parentStatus);
					if (parentStatusChanged == null && viewItem.parentStatus != parentStatus.value) {
						parentStatusChanged = true;
					}
					viewItem.parentStatus = (parentStatus != null) ? parentStatus.value : null;

					// Status css.
					statusCss = getStatusCss(parentStatus);
					viewItem.colour = statusCss.colour;
					viewItem.status = statusCss.status;
					viewItem.icon1 = statusCss.icon1;
					viewItem.icon2 = statusCss.icon2;

					// Display error if start time is not set.
					if (line.values.startTime == null) {
						lastError = FaxEnum.getLineLastErrorByValue(line.values.lastError);
						lastError = (lastError != null) ? Site.formatString("{0}: {1}", lastError.name, lastError.value) : null;
					}
					viewItem.error = lastError;

					// Pages sent.
					viewItem.pagesSent = line.values.pagesTransfered;

					// Total pages to be sent.
					viewItem.totalPages = line.values.totalPages;

					// Sent percentage.
					viewItem.sentPercent = (line.values.transferedPercent != null && status == LineStatus.SENDING) ? Site.formatString("{0}%", line.values.transferedPercent) : null;

					// Show pages.
					viewItem.showPages = !isNullOrEmpty(viewItem.pagesSent) && !isNullOrEmpty(viewItem.totalPages) && !isNullOrEmpty(viewItem.sentPercent);

					// Show transmission.
					viewItem.showTransmission = !isNullOrEmpty(viewItem.elapsedTime) || !isNullOrEmpty(viewItem.speed) || !isNullOrEmpty(viewItem.resolution) ||
                                            !isNullOrEmpty(viewItem.dataTransferred) || viewItem.showPages || !isNullOrEmpty(viewItem.error);

					// Show abort.
					viewItem.showAbort = ((line.values.caps & LineCapabilities.ABORT) == LineCapabilities.ABORT) &&
                                            (parentStatus == LineParentStatus.INBOUND || parentStatus == LineParentStatus.OUTBOUND);

					// Show initialise.
					viewItem.showInitialise = (line.values.caps & LineCapabilities.REINITIALISE) == LineCapabilities.REINITIALISE;
				});

				// If a parent status has changed, this means that a line is under another group so we need to update the sidebar counts.
				if (parentStatusChanged) {
					updateSidebarCounts(linesScope.items);
				}

				if (linesScope.sortBy != null) {
					//console.log(linesScope.sortBy + " | " + linesScope.isSortAsc);
					linesScope.items.sort(sortByProperty(linesScope.sortBy, linesScope.isSortAsc));
					linesScope.rebuildMappings();
				}

				// Update last sync identifier.
				lastSyncIdentifier = jsonObject.currentTimestamp;

				if (onUpdateUIComplete != null) {
					onUpdateUIComplete();
				}

				jsonObject = null;
				viewItem = null;
				status = null;
				parentStatus = null;
				resolution = null;
				statusCss = null;
				lastError = null;
				parentStatusChanged = null;
			});

			jqXHR.finally(function () {
				linesScope = null;
			});
		}
	};

	var updateLinesElapsedTime = function () {
		elapsedTimer = setTimeout(function () {
			var table = getLinesTable();
			var rows = table.find('tbody tr');
			var elementId;
			var angularItem;
			var linesScope = getLinesScope();

			if (linesScope != null) {
				safeApply(linesScope, function () {
					// Update elapsed time of all rows currently displayed on screen.
					$.each(rows, function (index, tmpRow) {
						elementId = '#' + tmpRow.id;
						angularItem = angular.element(elementId).scope().item;
						angularItem.elapsedTime = DateTime.getElapsedTime(angularItem.faxStartDateTime);
					});
				});
			}

			table = null;
			rows = null;
			elementId = null;
			angularItem = null;
			linesScope = null;

			updateLinesElapsedTime();
		}, elapsedTimeAutoRefreshTimeoutInSeconds * 1000);
	};

	var sortByProperty = function (property, isAsc) {
		return function (a, b) {
			try {
				var aValue, bValue;
				if (a[property] instanceof String || b[property] instanceof String) {
					aValue = a[property].toLowerCase();
					bValue = b[property].toLowerCase();
				} else {
					aValue = a[property];
					bValue = b[property];
				}

				if (isAsc == null || isAsc) {
					return ((aValue < bValue) ? -1 : ((aValue > bValue) ? 1 : 0));
				} else {
					return ((aValue > bValue) ? -1 : ((aValue < bValue) ? 1 : 0));
				}
			}
			catch (e) {
				console.log(e);
			}
		};
	};

	var getStatusCss = function (parentStatus) {
		var css = {};

		switch (parentStatus) {
			case LineParentStatus.FREE:
				css.colour = "green";
				css.status = "idle";
				css.icon1 = "icon icon-pictos-359";
				css.icon2 = "";
				break;
			case LineParentStatus.OUTBOUND:
				css.colour = "orange";
				css.status = "sending";
				css.icon1 = "icon icon-pictos-359";
				css.icon2 = "icon icon-pictos-454";
				break;
			case LineParentStatus.INBOUND:
				css.colour = "orange";
				css.status = "receiving";
				css.icon1 = "icon icon-pictos-359";
				css.icon2 = "icon icon-pictos-454";
				break;
			case LineParentStatus.UNINITIALISED:
			default:
				css.colour = "gray";
				css.status = "uninitialized";
				css.icon1 = "icon icon-pictos-641";
				css.icon2 = "";
				break;
		}

		return css;
	};

	var getFaxStartTime = function (serverTime, startTime) {
		if (serverTime == null || startTime == null
			|| serverTime == 0 || startTime == 0) {
			return null;
		}

		// Get the server DateTime.
		var serverDateTime = DateTime.getDateTimeFromEpoch(serverTime);

		// Get the DateTime when the line started processing.
		var lineStartDateTime = DateTime.getDateTimeFromEpoch(startTime);

		// Calculate the difference in milliseconds between the server DateTime and lineStartDateTime.
		var diffServerMilliseconds = serverDateTime.getTime() - lineStartDateTime.getTime();

		// Now that the server difference has been determined,
		// calculate the client difference.
		var diffClientMilliseconds = new Date() - diffServerMilliseconds;

		var clientStartDateTime = new Date(0); // The 0 sets the date to the epoch.
		clientStartDateTime.setUTCMilliseconds(diffClientMilliseconds);

		return clientStartDateTime;
	};

	var updateSidebarCounts = function (items) {
		var filteredItems;

		var sidebarScope = getSidebarScope();
		if (sidebarScope != null) {
			var sidebarItems = sidebarScope.items;

			safeApply(sidebarScope, function () {
				$.each(sidebarItems, function (index, sidebarItem) {
					filteredItems = getItemsByStatus(items, sidebarItem.parentStatus);

					sidebarItem.total = filteredItems.length;
				});
			});

			filteredItems = null;
			sidebarScope = null;
			sidebarItems = null;
		}
	};

	var getItemsByStatus = function (items, status) {
		if (isNullOrEmpty(status)) {
			return items;
		}

		// Filter items by status.
		var filtered = $.grep(items, function (item) {
			return item.parentStatus == status;
		});

		return filtered;
	};

	var getContainer = function () {
		return $('#wrapper');
	};

	// Get angular scope.
	var getNuggetsScope = function () {
		var dNuggets = $("#dNuggets");
		var scope = dNuggets.scope();

		dNuggets = null;
		element = null;

		return scope;
	};

	// Get angular scope.
	var getLinesScope = function () {
		var table = getLinesTable();
		var element = table.find('tbody');
		var scope = element.scope();

		table = null;
		element = null;

		return scope;
	};

	// Get angular scope.
	var getSidebarScope = function () {
		var element = $('#sidebarContainer');
		var scope = element.scope();

		element = null;

		return scope;
	};

	var getLinesTable = function () {
		return $('#tblLines');
	};

	var safeApply = function (scope, fn) {
		// Apply changes to UI.
		(scope.$$phase || scope.$root.$$phase) ? fn() : scope.$apply(fn);
	};

	var toDigits = function (value) {
		return value.replace(/(\d)(?=(\d\d\d)+(?!\d))/g, "$1,");
	};

	var resolveStorageSize = function (bytes) {
		var sizes = ["bytes", "KB", "MB", "GB", "TB"];
		var exponent = Math.min(sizes.length - 1, Math.log(bytes) / Math.LN2 / 10 | 0);
		var size = bytes / Math.pow(1024, exponent);

		// To 2 decimal places.
		size = size.toFixed(2);

		// If size ends with '00' remove it.
		if (size.substr(size.length - 2, 2) == 00) {
			size = size.substr(0, size.length - 3);
		}

		size = toDigits(size);

		return size + ' ' + sizes[exponent];
	};

	/**************************** Public functions ****************************/

	return {
		load: function () {
			HtmlStrings.load();

			init();
			updateUI();
			startPolling();
		},
		restartService: function () {
			var linesScope = getLinesScope();
			if (linesScope != null) {
				linesScope.restartService();
			}
		}
	};

}();